'use strict';

const rp = require('request-promise');
const { getRequestId, getAuthenticatedEntity } = require('./correlation');
const { REQUEST_ID_HEADER, AUTHENTICATED_ENTITY_HEADER } = require('./headers');

function request(opt) {
    opt = opt || {};

    const serviceConfig = opt.serviceConfig;
    if (!serviceConfig) {
        throw new Error('Service config not provided');
    }
    const serviceBasePath = `${serviceConfig.protocol}://${serviceConfig.uri}:${serviceConfig.port}`;
    const path = opt.path;
    const requestOptions = opt.requestOptions || {};
    const headers = Object.assign({}, requestOptions.headers);

    const authenticatedEntity = opt.authenticatedEntity || getAuthenticatedEntity();
    if (authenticatedEntity) {
        headers[AUTHENTICATED_ENTITY_HEADER] =
            authenticatedEntity.encodeToBase64String();
    }
    const requestId = getRequestId();
    if (requestId) {
        headers[REQUEST_ID_HEADER] = requestId;
    }

    const options = {
        url: `${serviceBasePath}${path}`,
        resolveWithFullResponse: true,
        json: true,
        headers,
    };

    return rp(Object.assign({}, requestOptions, options));
}

module.exports = request;
